@@ -202,6 +202,11 @@ void new_game (gchar* file, guint *seedp )
 
     game_file = file;
 
+    /* Explicitly drop any dragged cards. This shouldn't happen, but it
+     * has in the past so we should be defensive. See bug 121762. */
+    /* FIXME: we haven't actually fixed the cause of this. */
+    free_press_data ();
+    
     /* Although this line slows down game switching by a noticeable amount, we
      * add it here in order to make sure all the original functions are
      * "clean". */
@@ -397,7 +402,7 @@ static void create_press_data ()
   attributes.colormap = gdk_drawable_get_colormap (GDK_DRAWABLE(playing_area->window));
   attributes.visual = gdk_drawable_get_visual (GDK_DRAWABLE(playing_area->window));
   
-  press_data = malloc(sizeof(press_data_type));
+  press_data = g_malloc0(sizeof(press_data_type));
   press_data->moving_cards = gdk_window_new(playing_area->window, &attributes,
 					    (GDK_WA_VISUAL | GDK_WA_COLORMAP));
   press_data->status = 0;
