@@ -544,7 +544,7 @@ curtain_ready (ViewCurtainMode mode)
 	     CURTAIN_DISPLAY_NONE;
 
 	  state= STATE_WAITING_TO_START;
-	  gtk_timeout_add (START_DELAY, start_cave_delay_timeout, curtain_cave);
+	  g_timeout_add (START_DELAY, start_cave_delay_timeout, curtain_cave);
 	}
       else
 	{
@@ -556,7 +556,7 @@ curtain_ready (ViewCurtainMode mode)
 
     case VIEW_CURTAIN_CLOSED:
       /* If the iteration has been running, we stop it now.  */
-      if (iteration_timeout) gtk_timeout_remove (iteration_timeout);
+      if (iteration_timeout) g_source_remove (iteration_timeout);
       iteration_timeout=0;
       
       /* An existing cave must be deleted.  */
@@ -697,15 +697,15 @@ countdown_start (GStonesCave *cave)
 {
   /* When showing the countdown, there is no iteration in the
      background anymore.  */
-  if (iteration_timeout) gtk_timeout_remove (iteration_timeout);
+  if (iteration_timeout) g_source_remove (iteration_timeout);
   iteration_timeout=0;
 
   state    = STATE_COUNTDOWN;
   
   countdown_timeout= 
-    gtk_timeout_add (COUNTDOWN_DELAY, 
-		     (GtkFunction) countdown_timeout_function, 
-		     (gpointer) cave);
+    g_timeout_add (COUNTDOWN_DELAY, 
+		   (GSourceFunc) countdown_timeout_function, 
+		   (gpointer) cave);
 }
 
 
@@ -899,7 +899,7 @@ iteration_start (GStonesCave *newcave)
   status_set_cave (cave->name);
   
   iteration_timeout= 
-    gtk_timeout_add (game->frame_rate, iteration_timeout_function, cave);
+    g_timeout_add (game->frame_rate, iteration_timeout_function, cave);
 }
 
 
@@ -913,7 +913,6 @@ load_game (const gchar *filename, guint _start_cave)
   gchar *basename;
   char buffer[256];
   GStonesGame *newgame;
-  gint lastcave;
 
   if (!filename)
     return FALSE;
@@ -934,9 +933,9 @@ load_game (const gchar *filename, guint _start_cave)
   if (newgame)
     {
       /* Remove all running timeouts.  */
-      if (iteration_timeout) gtk_timeout_remove (iteration_timeout);
+      if (iteration_timeout) g_source_remove (iteration_timeout);
       iteration_timeout=0;
-      if (countdown_timeout) gtk_timeout_remove (countdown_timeout);
+      if (countdown_timeout) g_source_remove (countdown_timeout);
       countdown_timeout=0;
 
       /* Set display into title mode.  */
@@ -956,8 +955,7 @@ load_game (const gchar *filename, guint _start_cave)
          automatically, when starting a game.  */
       cave      = NULL;
       game      = newgame;
-      lastcave = g_list_length (game->start_caves) - 1;
-      start_cave = CLAMP (_start_cave, 0, lastcave);
+      start_cave= _start_cave;
 
       /* Update the title image.  */
       game_update_title ();
@@ -1049,13 +1047,13 @@ preferences_cb (GtkWidget *widget, gpointer data)
 }
 
 
-void
+static void
 quit_cb (GtkWidget *widget, gpointer data)
 {
   preferences_save_global ();
   sound_close();
 
-  gtk_main_quit ();
+  exit (0);
 }
 
 
@@ -1287,6 +1285,21 @@ static gint startup_function (gpointer data)
   return (index != max);
 }
 
+
+/************************************************************************/
+/* exit function */
+
+/* FIXME: too many different exit/quit functions */
+
+
+void
+gstones_exit (GnomeClient *client, gpointer client_data)
+{
+  sound_close (); 
+  exit (0);
+}
+
+
 
 /*****************************************************************************/
 /* Main function.  */
@@ -1374,7 +1387,9 @@ main (int argc, char *argv[])
   g_signal_connect (GTK_OBJECT (app), "key_release_event",
 		      GTK_SIGNAL_FUNC (game_widget_key_release_callback), 0);
   g_signal_connect (GTK_OBJECT (app), "delete_event",
-		      GTK_SIGNAL_FUNC (quit_cb), 0);
+		      GTK_SIGNAL_FUNC (gstones_exit), 0);
+  g_signal_connect (GTK_OBJECT (app), "destroy_event",
+		      GTK_SIGNAL_FUNC (gtk_main_quit), 0);
 
   joystick_set_widget (app);
 
@@ -1386,7 +1401,7 @@ main (int argc, char *argv[])
 
   session_management_init ();
 
-  gtk_idle_add (startup_function, NULL);
+  g_idle_add (startup_function, NULL);
 
   sound_init ();
 
