@@ -1,13 +1,28 @@
 /* -*- mode:C; tab-width:8; c-basic-offset:8; indent-tabs-mode:true -*- */
 
 /*
- *
+ * Mines for GNOME
  * Author:        Pista <szekeres@cyberspace.mht.bme.hu>
  *
  * Score support: horape@compendium.com.ar
  * Mine Resizing: djb@redhat.com
  *
+ * This game is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ * USA
  */
+
 #include <config.h>
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
@@ -72,6 +87,10 @@ char *fsize2names[] = {
 	N_("Custom"),
 };
 
+/* It's a little ugly, but it stops the hint dialogs triggering the
+ * hide-the-window-to-stop cheating thing. */
+gboolean disable_hiding = FALSE;
+
 static void create_preferences (void);
 
 
@@ -171,8 +190,6 @@ top_ten (GtkWidget *widget, gpointer data)
 static void
 new_game (GtkWidget *widget, gpointer data)
 {
-	static guint oldx = -1;
-	static guint oldy = -1;
 	gint width, height, w_diff, h_diff;
 	guint size;
 	gint x, y;
@@ -203,25 +220,48 @@ new_game (GtkWidget *widget, gpointer data)
 
 	set_flabel (GTK_MINEFIELD (mfield));
 
-	if ((oldx != -1) && (oldy != -1) &&
-	    ((oldx != x) || (oldy != y))) {
-		/* preserve the existing mine square size */
-		/* preserve any spacing around the minefield also */
-		width  = size * mf->xsize + w_diff;
-		height = size * mf->ysize + h_diff;
-		gtk_window_resize (GTK_WINDOW (window), width, height);
-	}
-	oldx = x;
-	oldy = y;
-	
 	gtk_widget_hide (ralign);
 	gtk_widget_show (mf_frame); 
 }
 
+static void
+hint (GtkWidget *widget, gpointer data)
+{
+	int result;
+	gchar * message;
+	GtkWidget * dialog;
+	
+	result = gtk_minefield_hint (GTK_MINEFIELD (mfield));
+
+	if (result == MINEFIELD_HINT_ACCEPTED) {
+		/* There is a ten second penalty for accepting a hint. */
+		games_clock_add_seconds (GAMES_CLOCK (clk), 10);
+		return;
+	}
+
+	if (result == MINEFIELD_HINT_NO_GAME)
+		message = _("Click a square, any square");
+	else
+		message = _("Maybe they're all mines ...");
+
+	dialog = gtk_message_dialog_new (GTK_WINDOW (window), GTK_DIALOG_MODAL,
+					 GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+					 message, NULL);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label), TRUE);
+
+	disable_hiding = TRUE;
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	disable_hiding = FALSE;
+
+}
+
 static void
 focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
-	if (GAMES_CLOCK (clk)->timer_id != -1) {
+	if ((GAMES_CLOCK (clk)->timer_id != -1) 
+	    && (!disable_hiding)) {
 		gtk_widget_hide (mf_frame);
 		gtk_widget_show (ralign);
 		gtk_widget_grab_focus (rbutton);
@@ -445,7 +485,7 @@ size_radio_callback (GtkWidget *widget, gpointer data)
 	fsc = GPOINTER_TO_INT (data);
 
 	gconf_client_set_int (conf_client, KEY_MODE,
-			fsc, NULL);
+			      fsc, NULL);
 
 	gtk_widget_set_sensitive(cframe, fsc == 3);
 }
@@ -453,17 +493,17 @@ size_radio_callback (GtkWidget *widget, gpointer data)
 static void
 fix_nmines (int xsize, int ysize)
 {
-  int maxmines;
+	int maxmines;
 
-  /* Fix up the maximum number of mines so that there is always at least two
-   * free spaces. It could in theory be left at one, but that gives an
-   * instant win situation. */
-  maxmines = xsize*ysize - 2;
-  if (nmines > maxmines) {
-    gconf_client_set_int (conf_client, KEY_NMINES, maxmines, NULL);
-    gtk_spin_button_set_value (GTK_SPIN_BUTTON(mentry), maxmines);
-  }
-  gtk_spin_button_set_range (GTK_SPIN_BUTTON(mentry), 1, maxmines);
+	/* Fix up the maximum number of mines so that there is always at least two
+	 * free spaces. It could in theory be left at one, but that gives an
+	 * instant win situation. */
+	maxmines = xsize*ysize - 2;
+	if (nmines > maxmines) {
+		gconf_client_set_int (conf_client, KEY_NMINES, maxmines, NULL);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (mentry), maxmines);
+	}
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (mentry), 1, maxmines);
 }
 
 static void
@@ -472,7 +512,7 @@ xsize_spin_cb (GtkSpinButton *spin, gpointer data)
 	int size = gtk_spin_button_get_value_as_int (spin);
 	gconf_client_set_int (conf_client, KEY_XSIZE,
 			      size, NULL);
-        fix_nmines(size,ysize);
+        fix_nmines (size,ysize);
 }
 
 static void
@@ -496,7 +536,7 @@ nmines_spin_cb (GtkSpinButton *spin, gpointer data)
 static void
 use_question_toggle_cb (GtkCheckButton *check, gpointer data)
 {
-	gboolean use_marks = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
+	gboolean use_marks = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
 	gconf_client_set_bool (conf_client, KEY_USE_QUESTION_MARKS,
 				use_marks, NULL);
 }
@@ -650,6 +690,7 @@ preferences_callback (GtkWidget *widget, gpointer data)
 
 GnomeUIInfo gamemenu[] = {
         GNOMEUIINFO_MENU_NEW_GAME_ITEM(new_game, NULL),
+	GNOMEUIINFO_MENU_HINT_ITEM(hint, NULL),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_SCORES_ITEM(top_ten, NULL),
 	GNOMEUIINFO_SEPARATOR,
@@ -722,9 +763,9 @@ static struct poptOption options[] = {
   {NULL, 'x', POPT_ARG_INT, &xsize, 0, N_("Width of grid"), N_("X")},
   {NULL, 'y', POPT_ARG_INT, &ysize, 0, N_("Height of grid"), N_("Y")},
   {NULL, 'n', POPT_ARG_INT, &nmines, 0, N_("Number of mines"), N_("NUMBER")},
-  {NULL, 'f', POPT_ARG_INT, &fsize, 0, NULL, NULL},
-  {NULL, 'a', POPT_ARG_INT, &xpos, 0, NULL, N_("X")},
-  {NULL, 'b', POPT_ARG_INT, &ypos, 0, NULL, N_("Y")},
+  {NULL, 'f', POPT_ARG_INT, &fsize, 0, N_("Size of the board (1=small, 3=large)"), NULL},
+  {NULL, 'a', POPT_ARG_INT, &xpos, 0, N_("X location of window"), N_("X")},
+  {NULL, 'b', POPT_ARG_INT, &ypos, 0, N_("Y location of window"), N_("Y")},
   {NULL, '\0', 0, NULL, 0}
 };
 
